@@ -25,8 +25,8 @@ typedef enum { CLICK_BORDER = 0, CLICK_DECORATION = 1, CLICK_INSIDE = 2 } click_
  * then calls resize_graphical_handler().
  *
  */
-static bool tiling_resize_for_border(Con *con, border_t border, const xcb_button_press_event_t *event) {
-    DLOG("border = %d\n", border);
+static bool tiling_resize_for_border(Con *con, border_t border, xcb_button_press_event_t *event) {
+    DLOG("border = %d, con = %p\n", border, con);
     char way = (border == BORDER_TOP || border == BORDER_LEFT ? 'p' : 'n');
     orientation_t orientation = (border == BORDER_TOP || border == BORDER_BOTTOM ? VERT : HORIZ);
 
@@ -38,6 +38,7 @@ static bool tiling_resize_for_border(Con *con, border_t border, const xcb_button
            resize_con->parent->orientation != orientation)
         resize_con = resize_con->parent;
 
+    DLOG("resize_con = %p\n", resize_con);
     if (resize_con->type != CT_WORKSPACE &&
         resize_con->type != CT_FLOATING_CON &&
         resize_con->parent->orientation == orientation) {
@@ -51,17 +52,25 @@ static bool tiling_resize_for_border(Con *con, border_t border, const xcb_button
             first = second;
             second = tmp;
         }
+        DLOG("first = %p, second = %p, resize_con = %p\n",
+                first, second, resize_con);
     }
 
     if (first == NULL || second == NULL) {
         DLOG("Resize not possible\n");
         return false;
     }
-    else {
-        assert(first != second);
-        assert(first->parent == second->parent);
-        resize_graphical_handler(first, second, orientation, event);
-    }
+
+    assert(first != second);
+    assert(first->parent == second->parent);
+
+    /* We modify the X/Y position in the event so that the divider line is at
+     * the actual position of the border, not at the position of the click. */
+    if (orientation == HORIZ)
+        event->root_x = second->rect.x;
+    else event->root_y = second->rect.y;
+
+    resize_graphical_handler(first, second, orientation, event);
 
     DLOG("After resize handler, rendering\n");
     tree_render();
@@ -76,7 +85,7 @@ static bool tiling_resize_for_border(Con *con, border_t border, const xcb_button
  * to the client).
  *
  */
-static bool floating_mod_on_tiled_client(Con *con, const xcb_button_press_event_t *event) {
+static bool floating_mod_on_tiled_client(Con *con, xcb_button_press_event_t *event) {
     /* The client is in tiling layout. We can still initiate a resize with the
      * right mouse button, by chosing the border which is the most near one to
      * the position of the mouse pointer */
@@ -115,7 +124,7 @@ static bool floating_mod_on_tiled_client(Con *con, const xcb_button_press_event_
  * Finds out which border was clicked on and calls tiling_resize_for_border().
  *
  */
-static bool tiling_resize(Con *con, const xcb_button_press_event_t *event, const click_destination_t dest) {
+static bool tiling_resize(Con *con, xcb_button_press_event_t *event, const click_destination_t dest) {
     /* check if this was a click on the window border (and on which one) */
     Rect bsr = con_border_style_rect(con);
     DLOG("BORDER x = %d, y = %d for con %p, window 0x%08x\n",
@@ -130,13 +139,14 @@ static bool tiling_resize(Con *con, const xcb_button_press_event_t *event, const
         /* Since the container might either be the child *or* already a split
          * container (in the case of a nested split container), we need to make
          * sure that we are dealing with the split container here. */
-        if (con_is_leaf(con) && con->parent->type == CT_CON)
-            con = con->parent;
+        Con *check_con = con;
+        if (con_is_leaf(check_con) && check_con->parent->type == CT_CON)
+            check_con = check_con->parent;
 
-        if ((con->layout == L_STACKED ||
-             con->layout == L_TABBED ||
-             con->orientation == HORIZ) &&
-            con_num_children(con) > 1) {
+        if ((check_con->layout == L_STACKED ||
+             check_con->layout == L_TABBED ||
+             check_con->orientation == HORIZ) &&
+            con_num_children(check_con) > 1) {
             DLOG("Not handling this resize, this container has > 1 child.\n");
             return false;
         }
@@ -162,7 +172,7 @@ static bool tiling_resize(Con *con, const xcb_button_press_event_t *event, const
  * functions for resizing/dragging.
  *
  */
-static int route_click(Con *con, const xcb_button_press_event_t *event, const bool mod_pressed, const click_destination_t dest) {
+static int route_click(Con *con, xcb_button_press_event_t *event, const bool mod_pressed, const click_destination_t dest) {
     DLOG("--> click properties: mod = %d, destination = %d\n", mod_pressed, dest);
     DLOG("--> OUTCOME = %p\n", con);
     DLOG("type = %d, name = %s\n", con->type, con->name);
@@ -244,7 +254,7 @@ static int route_click(Con *con, const xcb_button_press_event_t *event, const bo
     }
 
     /* 7: floating modifier pressed, initiate a resize */
-    if (mod_pressed && event->detail == 3) {
+    if (dest == CLICK_INSIDE && mod_pressed && event->detail == 3) {
         if (floating_mod_on_tiled_client(con, event))
             return 1;
     }
