@@ -544,16 +544,7 @@ int main(int argc, char *argv[]) {
 
     property_handlers_init();
 
-    /* Set up the atoms we support */
-    xcb_atom_t supported_atoms[] = {
-#define xmacro(atom) A_ ## atom,
-#include "atoms.xmacro"
-#undef xmacro
-    };
-    xcb_change_property(conn, XCB_PROP_MODE_REPLACE, root, A__NET_SUPPORTED, XCB_ATOM_ATOM, 32, 16, supported_atoms);
-    /* Set up the window manager’s name */
-    xcb_change_property(conn, XCB_PROP_MODE_REPLACE, root, A__NET_SUPPORTING_WM_CHECK, XCB_ATOM_WINDOW, 32, 1, &root);
-    xcb_change_property(conn, XCB_PROP_MODE_REPLACE, root, A__NET_WM_NAME, A_UTF8_STRING, 8, strlen("i3"), "i3");
+    ewmh_setup_hints();
 
     keysyms = xcb_key_symbols_alloc(conn);
 
